// Link Display
// $Author: Jeffrey J Weimer$
// SVN History: $Revision: 1362 $ on $Date: 2010-07-20 15:35:08 -0700 (Tue, 20 Jul 2010) $

#define English

#pragma rtGlobals=1
#pragma IgorVersion=6.20
#pragma version=2.2

#pragma ModuleName=LinkDisplay

#pragma hide=1

Static StrConstant thePackage="LinkDisplay"
//Static StrConstant thePackageFolder="root:Packages:LinkDisplay"	no package folder
Static StrConstant theProcedureFile = "LinkDisplay.ipf"
Static StrConstant thePackageInfo = "A set of functions to link and display sets of waves with a common x-wave"
Static StrConstant thePackageAuthor = "Jeffrey J Weimer"
Static Constant thePackageVersion = 2.2
Static Constant hasHelp = 1
//Static Constant removable = 1	no need to remove

// Package Parameters

Static StrConstant kLinkStart ="<link>"
Static StrConstant kLinkEnd = "</link>"
Static StrConstant kregEx = "<link>(.+)</link>"
Static StrConstant kregExUn = "(.*)<link>(.+)</link>\r(.*)"

Static StrConstant NoWave="No independent wave for link!"
Static StrConstant NoLinks="No links made!"
Static StrConstant NoYWaves="No list of waves to display!"
Static StrConstant NoGraph="No graph window for appending!"
Static StrConstant InvalidGraph="Append window is not a graph!"
Static StrConstant NoYWave="No dependent waves named "

// Package Functions

Function SetupLinkDisplay()

	// this is the location of the string key
	// replace MyPackageName with the name of your package
	
	SVAR/Z pt = root:Packages:PackageTools:Keys:LinkDisplay
	string theCmd

	if (!SVAR_Exists(pt))
		sprintf theCmd, "\"%s\"", thePackage
		sprintf theCmd, "%s,file=\"%s\"", theCmd, theProcedureFile
		sprintf theCmd, "%s,info=\"%s\"", theCmd, thePackageInfo
		sprintf theCmd, "%s,author=\"%s\"", theCmd, thePackageAuthor
		sprintf theCmd, "%s,version=%g", theCmd, thePackageVersion
		sprintf theCmd, "%s,hasHelp=%d", theCmd, hasHelp
		sprintf theCmd, "PackageSetup(%s)", theCmd
		Execute/P/Q/Z theCmd
	else
		sprintf theCmd, "\"%s\"", thePackage
		sprintf theCmd, "%s,version=%g", theCmd, thePackageVersion
		sprintf theCmd, "PackageUpdate(%s)", theCmd
		Execute/P/Q/Z theCmd
	endif
	return 0
end

// Link
// link waves across folders or within folder
// xwave: independent wave
// ywaveList: string list of dependent waves in current folder or via path
// full: use full paths (default or full = 1) or only partial paths (full = 0)

Static Function Link(xwave,ywaveList,[full])
	wave xwave
	string ywaveList
	variable full
	
	string theOne="", itsNote="", itsLink="", itsFldr=""
	variable ic, nt
	
	SetupLinkDisplay()
	
	if (!WaveExists(xwave))
		DoAlert 0, NoWave
		return -1
	endif
	
	if (ParamIsDefault(full))
		full = 1
	else
		if (cmpstr(GetDataFolder(1),GetWavesDataFolder(xwave,1))!=0)
			full = 1
		endif
	endif
	
	if (full)
		itsLink = GetWavesDataFolder(xwave,2)
	else
		itsLink = NameofWave(xwave)
	endif
	
	if (strlen(ywaveList)==0)
		ywaveList = RemoveFromList(NameofWave(xwave),WaveList("*",";",""))
		nt = ItemsInList(ywaveList)
		if (nt == 0)
			DoAlert 0, NoLinks
			return -1
		endif
		if (full)
			itsFldr = GetDataFolder(1)
			for (ic=0;ic<nt;ic+=1)
				sprintf theOne, "%s%s;" itsFldr, StringFromList(ic,ywaveList)
				itsNote += theOne
			endfor
			ywaveList = itsNote
		endif
	else
		nt = ItemsInList(ywaveList)
		for (ic=0;ic<nt;ic+=1)
			theOne = StringFromList(ic,ywaveList)
			if (!WaveExists($theOne))
				itsNote += theOne + ";"
			endif
		endfor
		if (strlen(itsNote)!=0)
			DoAlert 0, (NoYWave + itsNote)
			ywaveList = RemoveFromList(itsNote,ywaveList)
			nt = ItemsInList(ywaveList)
			if (nt ==0)
				DoAlert 0, NoLinks
				return -1
			endif
		endif
	endif
	
	// remove an existing link notation in the "xwave" if it exists
	
	theOne = GetWavesDataFolder(xwave,2)
	wave xwave = $theOne
	itsNote = note(xwave)
	SplitString/E=(kregEx) itsNote
	if (strlen(S_value)!=0)
		itsNote = ReplaceString((S_value+"\r"),itsNote,"")
		Note/K xwave, itsNote
	endif
	
	// add link notations to the waves
	
	nt = ItemsInList(ywaveList)
	for (ic=0;ic<nt;ic+=1)
		theOne = StringFromList(ic,ywaveList)
		wave ywave = $theOne
		itsNote = note(ywave)
		SplitString/E=(kregEx) itsNote
		if (strlen(S_value)!=0)
			sprintf theOne, "%s%s%s", kLinkStart,itsLink,kLinkEnd
			itsNote = ReplaceString(S_value,itsNote,theOne)
		else
			sprintf itsNote, "%s%s%s\r%s", kLinkStart,itsLink,kLinkEnd,itsNote
		endif
		Note/K ywave, itsNote
	endfor

	return 0
end

// UnLink
// unlink waves across folders or within folder
// ywaveList: string list of dependent waves in current folder or via path

Static Function UnLink(ywaveList)
	string ywaveList
	
	string theOne="", itsNote="", itsStart, theP, itsEnd
	variable ic, nt
	
	SetupLinkDisplay()
	
	if (strlen(ywaveList)==0)
		ywaveList = WaveList("*",";","")
	endif
	nt = ItemsInList(ywaveList)
	if (nt == 0)
		DoAlert 0, NoLinks
		return -1
	endif

	for (ic=0;ic<nt;ic+=1)
		theOne = StringFromList(ic,ywaveList)
		wave ywave = $theOne
		itsNote = note(ywave)
		SplitString/E=(kregExUn) itsNote, itsStart, theP, itsEnd
		itsNote = itsStart + itsEnd
		Note/K ywave, itsNote
	endfor

	return 0
end

// LDisplay
// display linked waves across folders or in folder
// ywaveList: string list of dependent waves in current folder or via path
// flags: string of flags to append to Display command
// title: string title for graph

Static Function LDisplay(ywaveList,[flags,title])
	string ywaveList, flags, title
	
	string theOne="", itsNote="", theXwave="", theCmd="", ptitle=""
	variable ic, nt, na
	
	// set the ywave list
	
	if (strlen(ywaveList)==0)
		ywaveList = WaveList("*",";","")
		nt = ItemsInList(ywaveList)
		if (nt ==0)
			DoAlert 0, NoYWaves
			return -1
		endif
		for(ic=0;ic<nt;ic+=1)
			theOne = StringFromList(ic,ywaveList)
			itsNote = note($theOne)
			SplitString/E=(kregEx) itsNote, theXwave
			if (strlen(S_Value)!=0)
				ywaveList = RemoveFromList(NameofWave($theXwave),ywaveList)
				break
			endif
		endfor
	else
		nt = ItemsInList(ywaveList)
		for (ic=0;ic<nt;ic+=1)
			theOne = StringFromList(ic,ywaveList)
			if (!WaveExists($theOne))
				itsNote += theOne + ";"
			endif
		endfor
		if (strlen(itsNote)!=0)
			DoAlert 0, (NoYWave + itsNote)
			ywaveList = RemoveFromList(itsNote,ywaveList)
			nt = ItemsInList(ywaveList)
			if (nt ==0)
				DoAlert 0, NoYWaves
				return -1
			endif
		endif
	endif
		
	// set the flags
	
	if (ParamIsDefault(flags))
		flags = ""
	endif
	
	// set default ptitle string
	
	if (!ParamIsDefault(title))
		if (strlen(title)!=0)
			title = ReplaceString("^",title,theXwave)
		endif
	else
		title = ""
	endif
	
	nt = ItemsInList(ywaveList)
	for (ic=0;ic<nt;ic+=1)
		theOne = StringFromList(ic,ywaveList)
		itsNote = note($theOne)
		SplitString/E=(kregEx) itsNote, theXWave
		if (strlen(S_Value)!=0)
			if (strlen(title)==0)
				if (!WaveExists($theXWave))
					sprintf theCmd "Display%s %s", flags, theOne
				else
					sprintf theCmd "Display%s %s vs %s", flags, theOne, theXwave
				endif
			else
				ptitle = ReplaceString("#",title,num2str(ic))
				ptitle = ReplaceString("&",ptitle,theOne)
				if (!WaveExists($theXWave))
					sprintf theCmd "Display%s %s as \"%s\"", flags, theOne, ptitle
				else
					sprintf theCmd "Display%s %s vs %s as \"%s\"", flags, theOne, theXwave, ptitle
				endif
			endif
		else
			if (strlen(title)==0)
				sprintf theCmd, "Display%s %s", flags, theOne
			else
				ptitle = ReplaceString("#",title,num2str(ic))
				ptitle = ReplaceString("&",ptitle,theOne)
				sprintf theCmd "Display%s %s as \"%s\"" flags, theOne, ptitle
			endif
		endif
		Execute/Q theCmd
	endfor
	killvariables/Z S_name
	return 0
end

// LAppendToGraph
// append linked waves across folders or in folder
// ywaveList: string list of dependent waves in current folder or via path
// win (optional): window where appending (default - frontmost)
// flags (optional): string list of options to append to appendtograph command (default - blank)

Static Function LAppendToGraph(ywaveList,[win, flags])
	string ywaveList, win, flags

	string theOne="", itsNote="", theXwave="", theCmd="", pwin=""
	variable nt, ic

	if (ParamIsDefault(flags))
		flags = ""
	endif
	
	// set wave list
	
	if (strlen(ywaveList)==0)
		ywaveList = WaveList("*",";","")
		nt = ItemsInList(ywaveList)
		if (nt ==0)
			DoAlert 0, NoYWaves
			return -1
		endif
		for(ic=0;ic<nt;ic+=1)
			theOne = StringFromList(ic,ywaveList)
			itsNote = note($theOne)
			SplitString/E=(kregEx) itsNote, theXwave
			if (strlen(S_Value)!=0)
				ywaveList = RemoveFromList(NameofWave($theXwave),ywaveList)
				break
			endif
		endfor
	else
		nt = ItemsInList(ywaveList)
		for (ic=0;ic<nt;ic+=1)
			theOne = StringFromList(ic,ywaveList)
			if (!WaveExists($theOne))
				itsNote += theOne + ";"
			endif
		endfor
		if (strlen(itsNote)!=0)
			DoAlert 0, (NoYWave + itsNote)
			ywaveList = RemoveFromList(itsNote,ywaveList)
			nt = ItemsInList(ywaveList)
			if (nt ==0)
				DoAlert 0, NoYWaves
				return -1
			endif
		endif
	endif
	
	// define append window
	
	if (ParamIsDefault(win))
	
		// default is frontmost or first indexed graph
		
		pwin = WinName(0,1)
		if (strlen(pwin)==0)
			DoAlert 0, NoGraph
			return -1
		endif

	else	
	
		// otherwise use win name (if it is a graph)
		
		if (WinType(win)!=1)
			DoAlert 0, InvalidGraph
			return -1
		else
			pwin = win
		endif
	endif
	
	// append to graph
	
	nt = ItemsInList(ywaveList)
	for (ic=0;ic<nt;ic+=1)
		theOne = StringFromList(ic,ywaveList)
		itsNote = note($theOne)
		SplitString/E=(kregEx) itsNote, theXwave
		if (strlen(S_Value)!=0)
			if (!WaveExists($theXwave))
				if (strlen(pwin)==0)
					sprintf theCmd,"appendtograph%s %s", flags, theOne
				else
					sprintf theCmd,"appendtograph/W=%s%s %s", pwin, flags, theOne
				endif
			else
				sprintf theCmd,"appendtograph%s %s vs %s", flags, theOne, theXwave
			endif
		else
			if (strlen(pwin)==0)
				sprintf theCmd,"appendtograph%s %s", flags, theOne
			else
				sprintf theCmd,"appendtograph/W=%s%s %s", pwin, flags, theOne
			endif
		endif
		Execute/Q theCmd
	endfor
	return 0
end

// DisplayVS(xwave,[ywaveList,flags])
// display linked waves in current folder
// xwave: wave name of xwave
// ywaveList (optional): string list of dependent waves in current folder (default - all others)
// flags (optional): string list of options to append to display command (default - none)
// title (optional): title for display (default - display default)

Static Function DisplayVS(xwave,[ywaveList,flags,title])
	wave xwave
	string ywaveList, flags, title
			
	// OPERATE WITHIN THE SAME DATA FOLDER AS XWAVE!

	DFREF xdf, cdf
	
	xdf = GetWavesDataFolderDFR(xwave)	
	cdf = GetDataFolderDFR()
	SetDataFolder xdf
	
	// link the files

	variable ret
	
	if (ParamIsDefault(ywaveList))
		ret = Link(xwave,"")	
		if (ret<0)
			return ret
		else
			ywaveList = ""
		endif
	else
		ret = Link(xwave,ywaveList)
		if (ret<0)
			return ret
		endif
	endif
	
	// display the waves
	
	if (ParamIsDefault(flags))
		if (ParamIsDefault(title))
			ret = LDisplay(ywaveList)
		else
			ret = LDisplay(ywaveList,title=title)
		endif
	else
		if (ParamIsDefault(title))
			ret = LDisplay(ywaveList,flags=flags)
		else
			ret = LDisplay(ywaveList,title=title,flags=flags)
		endif
	endif
	
	// WE NOW RETURN TO THE DATA FOLDER
	
	SetDataFolder cdf
	
	return ret
end

// AppendVS(xwave,[ywaveList, win, dflags, aflags,title])
// append linked waves in current folder
// xwave: wave name of xwave
// ywaveList (optional): string list of dependent waves in current folder (default - all others)
// win (optional): window where appending should happen (default - front, "" means new)
// dflags (optional): string list of options to append to display command (default - blank)
// aflags (optional): string list of options to append to appendtograph command (default - blank)
// title (optional): title for new display (default - display default)

Static Function AppendVS(xwave,[ywaveList,win,dflags,aflags,title])
	wave xwave
	string ywaveList, win, dflags, aflags, title
		
	// OPERATE WITHIN THE SAME DATA FOLDER AS XWAVE!

	DFREF xdf, cdf
	string theCmd
	
	xdf = GetWavesDataFolderDFR(xwave)	
	cdf = GetDataFolderDFR()
	SetDataFolder xdf
	
	// link the files

	variable ret
	
	if (ParamIsDefault(ywaveList))
		ret = Link(xwave,"")
	else
		ret = Link(xwave,ywaveList)
	endif
	
	if (ret<0)
		return ret
	endif
	
	// set default flags
	
	if (ParamIsDefault(dflags))
		dflags = ""
	endif
	if (ParamIsDefault(aflags))
		aflags = ""
	endif

	// do appending
	
	if (!ParamIsDefault(win))
		if (strlen(win)==0)
		
			// win = "" force creation of new graph
			
			if (ParamIsDefault(title))
				sprintf theCmd, "Display%s", dflags
			else
				sprintf theCmd, "Display%s as %s", dflags, title
			endif
			Execute/Q theCmd	
			if (ParamIsDefault(ywaveList))
				LAppendToGraph("",flags=aflags)
			else
				LAppendToGraph(ywaveList,flags=aflags)
			endif		
		else
		
			// win = something appends to something when it exists
			
			if (ParamIsDefault(ywaveList))
				LAppendToGraph("",flags=aflags,win=win)
			else
				LAppendToGraph(ywaveList,flags=aflags,win=win)
			endif
		endif
	else
	
		// win as default appends to frontmost (unless no existing graph)

		if (ParamIsDefault(ywaveList))
			LAppendToGraph("",flags=aflags)
		else
			LAppendToGraph(ywaveList,flags=aflags)
		endif
	endif
	
	// WE NOW RETURN TO THE DATA FOLDER
	
	SetDataFolder cdf

	return 0
end

// ItsLink
// returns the link wave name (as string) for a given wave

Static Function/S ItsLink(ywave)
	wave ywave
	
	string theXWave
	
	SplitString/E=(kregEx) note(ywave), theXWave
	
	return theXWave
end

// LinkList
// returns a list of waves in fldr according to
// 0: all waves in folder
// 1: all X waves (no link wavenotes) are enclosed in []
// -1: all X waves (no link wavenotes) are eliminated
// optional WavesList is used to limit sorting to only a selection of the given waves

Static Function/S LinkList(fldr,how,[WavesList])
	string fldr, WavesList
	variable how
	
	string wlist, olist, theLink, theOne, theSub
	variable ic, nt
	
	DFREF cdf
	cdf = GetDataFolderDFR()
	
	SetDatafolder $fldr

	if (ParamIsDefault(WavesList))
		wlist =  WaveList("*",";","DIMS:1")	
	else
		wlist = WavesList
	endif
	
	if (how==0)
		SetDataFolder cdf
		return wlist
	endif
	
	nt = ItemsInList(wlist)
	if (nt==0)
		SetDataFolder cdf
		return wlist
	endif
	
	olist = wlist
	if (nt>1)
		for (ic=0;ic<nt;ic+=1)
			theOne = StringFromList(ic,olist)
			theLink = ItsLink($theOne)
			if (strlen(theLink)!=0)
				theLink = ParseFilePath(3,theLink,":",0,0)
				sprintf theSub, "[%s]", theLink
				if (strsearch(wlist,theSub,0)<0)
					wlist = RemoveFromList(theLink,olist)
					wlist += theSub + ";"
				endif
			endif
		endfor
	endif
	
	SetDataFolder cdf

	if (how==-1)
		ic = 0
		do
			theOne = StringFromList(ic,wlist)
			if (strlen(theOne)==0)
				break
			endif
			if (strsearch(theOne,"[",0)==0)
				wlist = RemoveFromList(theOne,wlist)
			endif
			ic += 1
		while(1)
	endif
	
	return wlist
end
